#!/usr/bin/python

# IMPORT MODULES
import pygame,math,sys,random
from pygame.locals import *
from spaceship import SpaceShip
from bullet import Bullet
from enemy import Enemy


# INITIALIZE GAME ENVIRONMENT
pygame.init()
screen = pygame.display.set_mode((640, 480))
pygame.display.set_caption('ScribbleFighter')
clock = pygame.time.Clock()


# INITIALIZE SCORE
font = pygame.font.SysFont(None,30)
score = 0


# SET COLORS
black = (0,0,0)
white = (255,255,255)
red = (255,0,0)


# INITIALIZE PLAYER
s = SpaceShip([400,400])
ship_group = pygame.sprite.Group()
ship_group.add(s)

playerx = previous_playerx = 305
playery = previous_playery = 440
ship_velocity = 200

facing_left = False
facing_right = False


# INITIALIZE BULLETS
bullet_group = pygame.sprite.Group()

bullet_velocity = 350
bullet_timeout = 200
last_bullet = 0

# INITIALIZE ENEMIES
enemy_group = pygame.sprite.Group()
enemy_velocity = 100

enemy_timeout = 300
last_enemy = 0

# INITIALIZE KEYS
pressed_keys = {}
for key in [K_LEFT, K_RIGHT, K_UP, K_DOWN, K_SPACE, K_ESCAPE]:
	pressed_keys[key] = False


# DRAW GRAPHICS
ship_group.draw(screen)
bullet_group.draw(screen)
enemy_group.draw(screen)


# MAIN LOOP
while True:


	# START CLOCK
	time_passed = clock.tick()
	time_passed_secs = time_passed / 1000.0
	
	
	# READ KEY INPUT
	for event in pygame.event.get():
		if not hasattr(event, 'key'): continue
		if event.type == KEYDOWN: 
			pressed_keys[event.key] = True
		if event.type == KEYUP:
			pressed_keys[event.key] = False
	
	if pressed_keys[K_LEFT]:
		playerx += -ship_velocity * time_passed_secs
		facing_left = True
	if pressed_keys[K_RIGHT]:
		playerx += ship_velocity * time_passed_secs
		facing_right = True
		
	if pressed_keys[K_UP]: playery += -ship_velocity * time_passed_secs
	if pressed_keys[K_DOWN]: playery += ship_velocity * time_passed_secs

	if pressed_keys[K_SPACE]:
		if last_bullet > bullet_timeout:
			b = Bullet([playerx+2,playery])
			bullet_group.add(b)
#			b = Bullet([playerx+14,playery])
#			bullet_group.add(b)
			b = Bullet([playerx+26,playery])
			bullet_group.add(b)
			last_bullet = 0

	if pressed_keys[K_ESCAPE]: sys.exit(0)


	# CHECK BOUNDRIES
	if (5 < playerx < 605):
		playerx = playerx
	else:
		playerx = previous_playerx
	if (325 < playery < 450):
		playery = playery
	else:
		playery = previous_playery

	# CHECK DIRECTION
	if facing_left and not facing_right:
		ship_group.update([playerx,playery],"left")
	elif facing_right and not facing_left:
		ship_group.update([playerx,playery],"right")
	else:
		ship_group.update([playerx,playery],"straight")
		

	# CHECK COLLISION
	bullet_enemy_collide = pygame.sprite.groupcollide(bullet_group,enemy_group,True,True)
	ship_enemy_collide = pygame.sprite.groupcollide(ship_group,enemy_group,True,True)
	if len(ship_enemy_collide) != 0:
		break
	score += len(bullet_enemy_collide) * 5


	# UPDATE COORDINATES AND DIRECTIONS
	previous_playerx = playerx
	previous_playery = playery
	facing_left = False
	facing_right = False
	
	# NEW ENEMIES
	if last_enemy > enemy_timeout:
		start_enemyx = random.randrange(5,605)
		start_enemyy = 1
		e = Enemy([start_enemyx,start_enemyy])
		enemy_group.add(e)
		last_enemy = 0
	

	# UPDATE BULLETS
	bullet_group.update(time_passed_secs,bullet_velocity)
	last_bullet += time_passed

	# UPDATE ENEMIES
	enemy_group.update(time_passed_secs,enemy_velocity)
	last_enemy += time_passed

	# UPDATE SCORE
	scorePrint = "Score: " + str(score)
	scoreDisplay = font.render(scorePrint,True,(255,255,255))
	
	# FPS
	clockPrint = str(clock).split('(')
	clockPrint = clockPrint[1].replace(')>','')
	clockDisplay = font.render(clockPrint,True,(255,255,255))

	# UPDATE CANVAS
	screen.fill(black)
	enemy_group.draw(screen)
	bullet_group.draw(screen)
	ship_group.draw(screen)
	screen.fill(red,((0,0),(640,25)))
	screen.blit(scoreDisplay,(290,2))
	screen.blit(clockDisplay,(0,2))
	pygame.display.flip()
	
while True:
	for event in pygame.event.get():
		if event.type == KEYDOWN and event.key == K_ESCAPE:
			sys.exit(0)
			
	screen.fill(black)
	gameOver = font.render("Game Over",True,(255,255,255))
	screen.blit(scoreDisplay,(275,240))
	screen.blit(gameOver,(275,220))
	pygame.display.flip()